#
# Cambalache Type Info wrapper
#
# Copyright (C) 2021-2022  Juan Pablo Ugarte
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation;
# version 2.1 of the License.
#
# library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
# Authors:
#   Juan Pablo Ugarte <juanpablougarte@gmail.com>
#
# SPDX-License-Identifier: LGPL-2.1-only
#

from gi.repository import GObject, Gtk

from .cmb_base_objects import (
    CmbBaseTypeInfo,
    CmbBaseTypeDataInfo,
    CmbBaseTypeDataArgInfo,
    CmbBaseTypeInternalChildInfo,
    CmbTypeChildInfo,
    CmbSignalInfo,
)
from .cmb_property_info import CmbPropertyInfo

from .constants import EXTERNAL_TYPE, CUSTOM_TYPE, GMENU_TYPE, GMENU_SECTION_TYPE, GMENU_SUBMENU_TYPE, GMENU_ITEM_TYPE

from cambalache import getLogger

logger = getLogger(__name__)


class CmbTypeDataArgInfo(CmbBaseTypeDataArgInfo):
    def __init__(self, **kwargs):
        super().__init__(**kwargs)

    def __str__(self):
        return f"CmbTypeDataArgInfo<{self.owner_id}>::{self.key}"


class CmbTypeDataInfo(CmbBaseTypeDataInfo):
    def __init__(self, **kwargs):
        self.args = {}
        self.children = {}
        super().__init__(**kwargs)

    def __str__(self):
        return f"CmbTypeDataArgInfo<{self.owner_id}>::{self.key}"


class CmbTypeInternalChildInfo(CmbBaseTypeInternalChildInfo):
    def __init__(self, **kwargs):
        self.children = {}
        super().__init__(**kwargs)

    def __str__(self):
        return f"CmbTypeInternalChildInfo<{self.type_id}>::{self.internal_child_id}"


class CmbTypeInfo(CmbBaseTypeInfo):
    __gtype_name__ = "CmbTypeInfo"

    type_id = GObject.Property(type=str, flags=GObject.ParamFlags.READWRITE | GObject.ParamFlags.CONSTRUCT)
    parent_id = GObject.Property(type=str, flags=GObject.ParamFlags.READWRITE | GObject.ParamFlags.CONSTRUCT)
    parent = GObject.Property(type=GObject.Object, flags=GObject.ParamFlags.READWRITE)

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        if self.project is None:
            return

        self.hierarchy = self.__init_hierarchy()
        self.interfaces = self.__init_interfaces()
        self.properties = self.__init_properties_signals(CmbPropertyInfo, "property")
        self.signals = self.__init_properties_signals(CmbSignalInfo, "signal")
        self.data = self.__init_data()
        self.internal_children = self.__init_internal_children()
        self.child_constraint, self.child_type_shortcuts = self.__init_child_constraint()

        if self.parent_id == "enum":
            self.enum = self.__init_enum_flags("enum")
        elif self.parent_id == "flags":
            self.flags = self.__init_enum_flags("flags")

        self.child_types = self.__init_child_type()

        self.is_object = self.is_a("GObject")

        self.instantiable = self.is_object and not self.abstract

        self.is_menu_builtin = self.type_id in [GMENU_TYPE, GMENU_SECTION_TYPE, GMENU_SUBMENU_TYPE, GMENU_ITEM_TYPE]
        self.is_builtin = self.is_menu_builtin or self.type_id in [EXTERNAL_TYPE, CUSTOM_TYPE]

    def __str__(self):
        return f"CmbTypeInfo<{self.type_id}>"

    def __init_hierarchy(self):
        retval = []

        c = self.project.db.cursor()
        for row in c.execute(
            """
            WITH RECURSIVE ancestor(type_id, generation, parent_id) AS (
                SELECT type_id, 1, parent_id
                FROM type
                WHERE parent_id IS NOT NULL AND
                      parent_id != 'interface' AND
                      parent_id != 'enum' AND
                      parent_id != 'flags' AND
                      type_id=?
                UNION ALL
                SELECT ancestor.type_id, generation + 1, type.parent_id
                FROM type JOIN ancestor ON type.type_id = ancestor.parent_id
                WHERE type.parent_id IS NOT NULL AND type.parent_id != 'object' AND ancestor.type_id=?
            )
            SELECT parent_id, generation FROM ancestor
            UNION
            SELECT type_iface.iface_id, 0
            FROM ancestor JOIN type_iface
            WHERE ancestor.type_id = type_iface.type_id
            ORDER BY generation;
            """,
            (self.type_id, self.type_id),
        ):
            retval.append(row[0])

        c.close()

        return retval

    def __init_interfaces(self):
        retval = []

        c = self.project.db.cursor()
        for row in c.execute("SELECT iface_id FROM type_iface WHERE type_id=? ORDER BY iface_id;", (self.type_id,)):
            retval.append(row[0])

        c.close()
        return retval

    def __init_properties_signals(self, Klass, table):
        retval = {}

        c = self.project.db.cursor()
        for row in c.execute(f"SELECT * FROM {table} WHERE owner_id=? ORDER BY {table}_id;", (self.type_id,)):
            retval[row[1]] = Klass.from_row(self.project, *row)

        c.close()
        return retval

    def __type_get_data(self, owner_id, data_id, parent_id, key, type_id, translatable):
        args = {}
        children = {}
        parent_id = parent_id if parent_id is not None else 0
        retval = CmbTypeDataInfo.from_row(self.project, owner_id, data_id, parent_id, key, type_id, translatable)

        c = self.project.db.cursor()

        # Collect Arguments
        for row in c.execute("SELECT * FROM type_data_arg WHERE owner_id=? AND data_id=?;", (owner_id, data_id)):
            _key = row[2]
            args[_key] = CmbTypeDataArgInfo.from_row(self.project, *row)

        # Recurse children
        for row in c.execute("SELECT * FROM type_data WHERE owner_id=? AND parent_id=?;", (owner_id, data_id)):
            _key = row[3]
            children[_key] = self.__type_get_data(*row)

        c.close()

        retval.args = args
        retval.children = children

        return retval

    def __init_data(self):
        retval = {}

        c = self.project.db.cursor()
        for row in c.execute(
            "SELECT * FROM type_data WHERE parent_id IS NULL AND owner_id=? ORDER BY data_id;", (self.type_id,)
        ):
            key = row[3]
            retval[key] = self.__type_get_data(*row)

        c.close()
        return retval

    def __type_get_internal_child(self, type_id, internal_child_id, internal_parent_id, internal_type, creation_property_id):
        retval = CmbTypeInternalChildInfo.from_row(
            self.project,
            type_id,
            internal_child_id,
            internal_parent_id,
            internal_type,
            creation_property_id
        )
        children = {}

        c = self.project.db.cursor()

        # Recurse children
        for row in c.execute(
            "SELECT * FROM type_internal_child WHERE type_id=? AND internal_parent_id=?;",
            (type_id, internal_child_id)
        ):
            key = row[1]
            children[key] = self.__type_get_internal_child(*row)

        c.close()

        retval.children = children

        # Internal child back reference in property
        if creation_property_id:
            if creation_property_id in self.properties:
                self.properties[creation_property_id].internal_child = retval

        return retval

    def __init_internal_children(self):
        retval = {}

        c = self.project.db.cursor()
        for row in c.execute(
            "SELECT * FROM type_internal_child WHERE type_id=? AND internal_parent_id IS NULL ORDER BY internal_child_id;",
            (self.type_id,)
        ):
            key = row[1]
            retval[key] = self.__type_get_internal_child(*row)

        c.close()
        return retval

    def __init_child_constraint(self):
        retval = {}
        shortcuts = []

        c = self.project.db.cursor()
        for row in c.execute(
            "SELECT child_type_id, allowed, shortcut FROM type_child_constraint WHERE type_id=?;", (self.type_id,)
        ):
            child_type_id, allowed, shortcut = row
            retval[child_type_id] = allowed
            if shortcut:
                shortcuts.append(child_type_id)

        c.close()
        return retval, shortcuts

    def __init_child_type(self):
        retval = {}

        c = self.project.db.cursor()
        for row in c.execute("SELECT * FROM type_child_type WHERE type_id=?;", (self.type_id,)):
            type_id, child_type, max_children, linked_property_id = row
            retval[child_type] = CmbTypeChildInfo(
                project=self.project,
                type_id=type_id,
                child_type=child_type,
                max_children=max_children if max_children else 0,
                linked_property_id=linked_property_id,
            )

        c.close()
        return retval

    def __init_enum_flags(self, name):
        retval = Gtk.ListStore(GObject.TYPE_STRING, GObject.TYPE_STRING, GObject.TYPE_INT)

        c = self.project.db.cursor()
        for row in c.execute(f"SELECT name, nick, value FROM type_{name} WHERE type_id=? ORDER BY nick;", (self.type_id,)):
            retval.append(row)

        c.close()
        return retval

    def is_a(self, type_id):
        return self.type_id == type_id or type_id in self.hierarchy

    def get_data_info(self, name):
        parent = self
        while parent:
            if name in parent.data:
                return parent.data[name]

            parent = parent.parent

        return None

    def find_data_info(self, data_id):
        def find_child_info(info, data_id):
            for name in info.children:
                child_info = info.children[name]
                if child_info.data_id == data_id:
                    return child_info

                retval = find_child_info(child_info, data_id)
                if retval:
                    return retval

        for name in self.data:
            info = self.data[name]
            if info.data_id == data_id:
                return info

            retval = find_child_info(info, data_id)
            if retval:
                return retval

        return None

    def has_child_types(self):
        parent = self
        while parent:
            if parent.child_types:
                return True
            parent = parent.parent

        return False

    def enum_get_value_as_string(self, value, use_nick=True):
        if self.parent_id != "enum":
            return None

        for row in self.enum:
            enum_name, enum_nick, enum_value = row

            # Always use nick as value
            if value == enum_name or value == enum_nick or value == str(enum_value):
                return enum_nick if use_nick else enum_value

        return None

    def flags_get_value_as_string(self, value):
        if self.parent_id != "flags":
            return None

        value_type = type(value)
        tokens = None

        if value_type == str:
            if value.isnumeric():
                value = int(value)
                value_type = int
            else:
                tokens = [t.strip() for t in value.split("|")]
        elif value_type != int:
            logger.warning(f"Unhandled value type {value_type} {value}")
            return None

        flags = []

        for row in self.flags:
            flag_name, flag_nick, flag_value = row

            if value_type == str:
                # Always use nick as value
                if flag_name in tokens or flag_nick in tokens:
                    flags.append(flag_nick)
            else:
                if flag_value & value:
                    flags.append(flag_nick)

        return "|".join(flags)



import sys
import gi

gi.require_version("Gtk", "4.0")
from gi.repository import GLib, Gtk  # noqa: E402


class Preview(Gtk.Application):
    def __init__(self, filename):
        self.filename = filename

        super().__init__(application_id="ar.xjuan.CmbPreview")
        GLib.set_application_name("Cmb Simple previewer")

    def do_activate(self):
        builder = Gtk.Builder()
        builder.add_from_file(self.filename)

        for w in builder.get_objects():
            if isinstance(w, Gtk.Window):
                self.add_window(w)
                w.present()


if __name__ == "__main__":
    if len(sys.argv) != 2:
        print(f"Usage: {sys.argv[0]} file.ui")
        exit()

    app = Preview(sys.argv[1])
    app.run()



#
# CmbTypeChooserBar - Cambalache Type Chooser Bar
#
# Copyright (C) 2021-2024  Juan Pablo Ugarte
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation;
# version 2.1 of the License.
#
# library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
# Authors:
#   Juan Pablo Ugarte <juanpablougarte@gmail.com>
#
# SPDX-License-Identifier: LGPL-2.1-only
#

from gi.repository import GObject, Gtk

from .cmb_project import CmbProject
from .cmb_type_info import CmbTypeInfo


@Gtk.Template(resource_path="/ar/xjuan/Cambalache/cmb_type_chooser.ui")
class CmbTypeChooser(Gtk.Box):
    __gtype_name__ = "CmbTypeChooser"

    __gsignals__ = {
        "type-selected": (GObject.SignalFlags.RUN_LAST, None, (CmbTypeInfo,)),
        "chooser-popup": (GObject.SignalFlags.RUN_LAST, None, (GObject.Object,)),
        "chooser-popdown": (GObject.SignalFlags.RUN_LAST, None, (GObject.Object,)),
    }

    project = GObject.Property(type=CmbProject, flags=GObject.ParamFlags.READWRITE)
    selected_type = GObject.Property(type=CmbTypeInfo, flags=GObject.ParamFlags.READWRITE)

    type_label = Gtk.Template.Child()
    content = Gtk.Template.Child()
    all = Gtk.Template.Child()
    toplevel = Gtk.Template.Child()
    layout = Gtk.Template.Child()
    control = Gtk.Template.Child()
    display = Gtk.Template.Child()
    model = Gtk.Template.Child()
    extra = Gtk.Template.Child()

    def __init__(self, **kwargs):
        super().__init__(**kwargs)

        self._choosers = [self.all, self.toplevel, self.layout, self.control, self.display, self.model, self.extra]

        self.connect("notify::project", self.__on_project_notify)
        self.connect("notify::selected-type", self.__on_selected_type_notify)

        for chooser in self._choosers:
            chooser.connect("type-selected", lambda o, t: self.emit("type-selected", t))
            chooser.connect("notify::visible", self.__on_chooser_visible_notify)

    def __on_project_notify(self, object, pspec):
        project = self.project
        self.selected_type = None

        for chooser in self._choosers:
            chooser.project = project

    def __on_selected_type_notify(self, object, pspec):
        project_target = self.project.target_tk if self.project else ""
        self.type_label.props.label = self.selected_type.type_id if self.selected_type else project_target

    def __on_chooser_visible_notify(self, obj, pspec):
        if obj.props.visible:
            self.emit("chooser-popup", obj)
        else:
            self.emit("chooser-popdown", obj)

    def select_type_id(self, type_id):
        info = self.project.type_info.get(type_id, None)
        if info:
            self.selected_type = info
            self.emit("type-selected", info)


Gtk.WidgetClass.set_css_name(CmbTypeChooser, "CmbTypeChooser")



#
# MrgPlaceholder
#
# Copyright (C) 2021  Juan Pablo Ugarte
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation;
# version 2.1 of the License.
#
# library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
# Authors:
#   Juan Pablo Ugarte <juanpablougarte@gmail.com>
#
# SPDX-License-Identifier: LGPL-2.1-only
#

from gi.repository import GObject, Gtk

from .mrg_controller import MrgController
from merengue import getLogger

logger = getLogger(__name__)


class MrgPlaceholder(Gtk.Box):
    __gtype_name__ = "MrgPlaceholder"

    controller = GObject.Property(type=MrgController, flags=GObject.ParamFlags.READWRITE)

    def __init__(self, **kwargs):
        self.__binding = None

        super().__init__(**kwargs)

        self.props.can_focus = True
        self.props.hexpand = True
        self.props.vexpand = True
        self.set_size_request(32, 32)

        self.connect("notify::controller", self.__on_controller_notify)
        self.__update_preview_binding()

    def __on_controller_notify(self, obj, pspec):
        self.__update_preview_binding()

    def __update_preview_binding(self):
        if self.__binding:
            self.__binding.unbind()
            self.__binding = None

        if not self.controller:
            return

        self.__binding = GObject.Object.bind_property(
            self.controller.app,
            "preview",
            self,
            "visible",
            GObject.BindingFlags.SYNC_CREATE | GObject.BindingFlags.BIDIRECTIONAL | GObject.BindingFlags.INVERT_BOOLEAN,
        )

    def selected(self):
        if self.controller:
            self.controller.placeholder_selected(self)

    def activated(self):
        if self.controller:
            self.controller.placeholder_activated(self)


Gtk.WidgetClass.set_css_name(MrgPlaceholder, "MrgPlaceholder")



#
# CmbUIEditor - Cambalache UI Editor
#
# Copyright (C) 2021-2024  Juan Pablo Ugarte
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation;
# version 2.1 of the License.
#
# library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
# Authors:
#   Juan Pablo Ugarte <juanpablougarte@gmail.com>
#
# SPDX-License-Identifier: LGPL-2.1-only
#

import os

from gi.repository import GObject, Gtk

from cambalache import _
from .cmb_ui import CmbUI


@Gtk.Template(resource_path="/ar/xjuan/Cambalache/cmb_ui_property_editor.ui")
class CmbUIPropertyEditor(Gtk.Grid):
    __gtype_name__ = "CmbUIPropertyEditor"

    filename = Gtk.Template.Child()
    format = Gtk.Template.Child()
    template_id = Gtk.Template.Child()
    description = Gtk.Template.Child()
    copyright = Gtk.Template.Child()
    authors = Gtk.Template.Child()
    translation_domain = Gtk.Template.Child()
    comment = Gtk.Template.Child()

    fields = ["filename", "template_id", "description", "copyright", "authors", "translation_domain", "comment"]

    def __init__(self, **kwargs):
        self._object = None
        self._bindings = []

        super().__init__(**kwargs)

    @GObject.Property(type=CmbUI)
    def object(self):
        return self._object

    @object.setter
    def _set_object(self, obj):
        for binding in self._bindings:
            binding.unbind()

        self._bindings = []

        self._object = obj

        if obj is None:
            self.set_sensitive(False)
            for field in self.fields:
                widget = getattr(self, field)

                if type(widget.cmb_value) is int:
                    widget.cmb_value = 0
                else:
                    widget.cmb_value = None
            return

        self.set_sensitive(True)
        self.template_id.object = obj
        self.filename.dirname = obj.project.dirname

        # Set some default name
        self.filename.unnamed_filename = _("unnamed.ui")
        if not obj.filename and obj.template_id:
            template = obj.project.get_object_by_id(obj.ui_id, obj.template_id)
            if template:
                self.filename.unnamed_filename = f"{template.name}.ui".lower()

        for field in self.fields:
            binding = obj.bind_property(
                field,
                getattr(self, field),
                "cmb-value",
                GObject.BindingFlags.SYNC_CREATE | GObject.BindingFlags.BIDIRECTIONAL,
            )
            self._bindings.append(binding)

        if obj.project.target_tk == "gtk-4.0":
            self.filename.mime_types = "application/x-gtk-builder;text/x-blueprint"

            # filename -> format
            binding = obj.bind_property(
                "filename",
                self.format,
                "selected",
                GObject.BindingFlags.SYNC_CREATE | GObject.BindingFlags.BIDIRECTIONAL,
                transform_to=self.__filename_to_format,
                transform_from=self.__format_to_filename,
                user_data=obj
            )
            self._bindings.append(binding)

            self.format.show()
            self.format.set_sensitive(bool(obj.filename))
        else:
            self.filename.mime_types = "application/x-gtk-builder;application/x-glade"
            self.format.hide()

    def __filename_to_format(self, binding, source_value, ui):
        if not source_value:
            self.format.props.sensitive = False
            return 0
        self.format.props.sensitive = True

        return 1 if source_value.endswith(".blp") else 0

    def __format_to_filename(self, binding, target_value, ui):
        if not ui.filename:
            self.format.props.sensitive = False
            return None
        self.format.props.sensitive = True

        return os.path.splitext(ui.filename)[0] + (".blp" if target_value == 1 else ".ui")


Gtk.WidgetClass.set_css_name(CmbUIPropertyEditor, "CmbUIPropertyEditor")



#
# CmbPollNotificationView
#
# Copyright (C) 2025  Juan Pablo Ugarte
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation;
# version 2.1 of the License.
#
# library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
# Authors:
#   Juan Pablo Ugarte <juanpablougarte@gmail.com>
#
# SPDX-License-Identifier: LGPL-2.1-only
#

import datetime

from cambalache import _, getLogger
from gi.repository import GObject, Gtk
from . import utils
from .cmb_notification import CmbPollNotification
from .cmb_poll_option_check import CmbPollOptionCheck

logger = getLogger(__name__)


@Gtk.Template(resource_path="/ar/xjuan/Cambalache/cmb_poll_notification_view.ui")
class CmbPollNotificationView(Gtk.Box):
    __gtype_name__ = "CmbPollNotificationView"

    notification = GObject.Property(
        type=CmbPollNotification, flags=GObject.ParamFlags.READWRITE | GObject.ParamFlags.CONSTRUCT_ONLY
    )

    # Poll
    title_label = Gtk.Template.Child()
    description_label = Gtk.Template.Child()
    option_box = Gtk.Template.Child()
    total_label = Gtk.Template.Child()
    end_date_label = Gtk.Template.Child()
    refresh_button = Gtk.Template.Child()

    def __init__(self, **kwargs):
        self.__option_checks = []
        self.__updating = False
        super().__init__(**kwargs)

        notification = self.notification
        poll = notification.poll
        active = utils.utcnow() < poll.end_date

        self.title_label.props.label = f"<b>{poll.title}</b>"
        self.description_label.props.label = poll.description

        close_msg = _("<small>• Closes on {date}</small>") if active else _("<small>• Closed on {date}</small>")
        end_date = datetime.datetime.fromtimestamp(poll.end_date)
        self.end_date_label.props.label = close_msg.format(date=end_date.strftime("%x"))
        self.end_date_label.props.tooltip_text = end_date.strftime("%c")

        first_check = None
        n_option = 0
        for option in poll.options:
            button = CmbPollOptionCheck(option=option, sensitive=active)

            if poll.allowed_votes == 1:
                if first_check is None:
                    first_check = button
                else:
                    button.set_group(first_check)

            button.connect("toggled", self.__on_check_button_toggled, n_option)

            self.__option_checks.append(button)
            self.option_box.append(button)
            n_option += 1

        self.__update_results()
        notification.connect("notify", self.__on_poll_notify)

    def __on_check_button_toggled(self, button, n_option):
        allowed_votes = self.notification.poll.allowed_votes

        if self.__updating or (allowed_votes == 1 and not button.props.active):
            return

        votes = []
        for i, check in enumerate(self.__option_checks):
            if check.props.active:
                votes.append(i)

        if allowed_votes > 1:
            not_done = len(votes) < allowed_votes
            for i, check in enumerate(self.__option_checks):
                if not_done:
                    check.set_sensitive(True)
                elif not check.props.active:
                    check.set_sensitive(False)

        self.notification.center.poll_vote(self.notification, votes)

    def __on_poll_notify(self, notification, pspec):
        if pspec.name in ["my-votes", "results"]:
            self.__update_results()

    def __update_results(self):
        notification = self.notification
        results = notification.results
        my_votes = notification.my_votes

        if not results or not my_votes:
            self.total_label.props.label = ""
            for check in self.__option_checks:
                check.fraction = -1
            return

        self.__updating = True

        votes = results.votes
        total = results.total

        for i, check in enumerate(self.__option_checks):
            check.set_active(i in my_votes)
            check.fraction = votes[i] / total if total else 0

        self.total_label.props.label = _("<small>• {total} vote</small>").format(total=results.total)

        self.__updating = False

    @Gtk.Template.Callback("on_refresh_button_clicked")
    def __on_refresh_button_clicked(self, button):
        self.notification.center.poll_refresh_results(self.notification)




# flake8: noqa
# THIS FILE IS AUTOGENERATED, DO NOT EDIT!!!
#
# Cambalache Base Object wrappers
#
# Copyright (C) 2021-2024  Juan Pablo Ugarte
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation;
# version 2.1 of the License.
#
# library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
# Authors:
#   Juan Pablo Ugarte <juanpablougarte@gmail.com>
#
# SPDX-License-Identifier: LGPL-2.1-only
#

from gi.repository import GObject
from .cmb_base import CmbBase
from .cmb_base_file_monitor import CmbBaseFileMonitor


class CmbBaseLibraryInfo(CmbBase):
    __gtype_name__ = "CmbBaseLibraryInfo"

    library_id = GObject.Property(type=str, flags=GObject.ParamFlags.READWRITE | GObject.ParamFlags.CONSTRUCT_ONLY)

    def __init__(self, **kwargs):
        super().__init__(**kwargs)

    @classmethod
    def from_row(
        cls, project, library_id, version, namespace, prefix, shared_library, license_id, license_text, third_party, enabled
    ):
        return cls(project=project, library_id=library_id)

    @GObject.Property(type=str)
    def version(self):
        return self.db_get("SELECT version FROM library WHERE (library_id) IS (?);", (self.library_id,))

    @version.setter
    def _set_version(self, value):
        self.db_set("UPDATE library SET version=? WHERE (library_id) IS (?);", (self.library_id,), value)

    @GObject.Property(type=str)
    def namespace(self):
        return self.db_get("SELECT namespace FROM library WHERE (library_id) IS (?);", (self.library_id,))

    @namespace.setter
    def _set_namespace(self, value):
        self.db_set("UPDATE library SET namespace=? WHERE (library_id) IS (?);", (self.library_id,), value)

    @GObject.Property(type=str)
    def prefix(self):
        return self.db_get("SELECT prefix FROM library WHERE (library_id) IS (?);", (self.library_id,))

    @prefix.setter
    def _set_prefix(self, value):
        self.db_set("UPDATE library SET prefix=? WHERE (library_id) IS (?);", (self.library_id,), value)

    @GObject.Property(type=str)
    def shared_library(self):
        return self.db_get("SELECT shared_library FROM library WHERE (library_id) IS (?);", (self.library_id,))

    @shared_library.setter
    def _set_shared_library(self, value):
        self.db_set("UPDATE library SET shared_library=? WHERE (library_id) IS (?);", (self.library_id,), value)

    @GObject.Property(type=str)
    def license_id(self):
        return self.db_get("SELECT license_id FROM library WHERE (library_id) IS (?);", (self.library_id,))

    @license_id.setter
    def _set_license_id(self, value):
        self.db_set("UPDATE library SET license_id=? WHERE (library_id) IS (?);", (self.library_id,), value)

    @GObject.Property(type=str)
    def license_text(self):
        return self.db_get("SELECT license_text FROM library WHERE (library_id) IS (?);", (self.library_id,))

    @license_text.setter
    def _set_license_text(self, value):
        self.db_set("UPDATE library SET license_text=? WHERE (library_id) IS (?);", (self.library_id,), value)

    @GObject.Property(type=bool, default=False)
    def third_party(self):
        return self.db_get("SELECT third_party FROM library WHERE (library_id) IS (?);", (self.library_id,))

    @third_party.setter
    def _set_third_party(self, value):
        self.db_set("UPDATE library SET third_party=? WHERE (library_id) IS (?);", (self.library_id,), value)

    @GObject.Property(type=bool, default=False)
    def enabled(self):
        return self.db_get("SELECT enabled FROM library WHERE (library_id) IS (?);", (self.library_id,))

    @enabled.setter
    def _set_enabled(self, value):
        self.db_set("UPDATE library SET enabled=? WHERE (library_id) IS (?);", (self.library_id,), value)


class CmbBasePropertyInfo(CmbBase):
    __gtype_name__ = "CmbBasePropertyInfo"

    owner_id = GObject.Property(type=str, flags=GObject.ParamFlags.READWRITE | GObject.ParamFlags.CONSTRUCT_ONLY)
    property_id = GObject.Property(type=str, flags=GObject.ParamFlags.READWRITE | GObject.ParamFlags.CONSTRUCT_ONLY)
    type_id = GObject.Property(type=str, flags=GObject.ParamFlags.READWRITE | GObject.ParamFlags.CONSTRUCT_ONLY)
    is_object = GObject.Property(
        type=bool, flags=GObject.ParamFlags.READWRITE | GObject.ParamFlags.CONSTRUCT_ONLY, default=False
    )
    construct_only = GObject.Property(
        type=bool, flags=GObject.ParamFlags.READWRITE | GObject.ParamFlags.CONSTRUCT_ONLY, default=False
    )
    save_always = GObject.Property(
        type=bool, flags=GObject.ParamFlags.READWRITE | GObject.ParamFlags.CONSTRUCT_ONLY, default=False
    )
    default_value = GObject.Property(type=str, flags=GObject.ParamFlags.READWRITE | GObject.ParamFlags.CONSTRUCT_ONLY)
    minimum = GObject.Property(type=str, flags=GObject.ParamFlags.READWRITE | GObject.ParamFlags.CONSTRUCT_ONLY)
    maximum = GObject.Property(type=str, flags=GObject.ParamFlags.READWRITE | GObject.ParamFlags.CONSTRUCT_ONLY)
    version = GObject.Property(type=str, flags=GObject.ParamFlags.READWRITE | GObject.ParamFlags.CONSTRUCT_ONLY)
    deprecated_version = GObject.Property(type=str, flags=GObject.ParamFlags.READWRITE | GObject.ParamFlags.CONSTRUCT_ONLY)
    translatable = GObject.Property(
        type=bool, flags=GObject.ParamFlags.READWRITE | GObject.ParamFlags.CONSTRUCT_ONLY, default=False
    )
    disable_inline_object = GObject.Property(
        type=bool, flags=GObject.ParamFlags.READWRITE | GObject.ParamFlags.CONSTRUCT_ONLY, default=False
    )
    deprecated = GObject.Property(type=str, flags=GObject.ParamFlags.READWRITE | GObject.ParamFlags.CONSTRUCT_ONLY)
    required = GObject.Property(
        type=bool, flags=GObject.ParamFlags.READWRITE | GObject.ParamFlags.CONSTRUCT_ONLY, default=False
    )
    workspace_default = GObject.Property(type=str, flags=GObject.ParamFlags.READWRITE | GObject.ParamFlags.CONSTRUCT_ONLY)
    original_owner_id = GObject.Property(type=str, flags=GObject.ParamFlags.READWRITE | GObject.ParamFlags.CONSTRUCT_ONLY)
    disabled = GObject.Property(
        type=bool, flags=GObject.ParamFlags.READWRITE | GObject.ParamFlags.CONSTRUCT_ONLY, default=False
    )

    def __init__(self, **kwargs):
        super().__init__(**kwargs)

    @classmethod
    def from_row(
        cls,
        project,
        owner_id,
        property_id,
        type_id,
        is_object,
        construct_only,
        save_always,
        default_value,
        minimum,
        maximum,
        version,
        deprecated_version,
        translatable,
        disable_inline_object,
        deprecated,
        required,
        workspace_default,
        original_owner_id,
        disabled,
    ):
        return cls(
            project=project,
            owner_id=owner_id,
            property_id=property_id,
            type_id=type_id,
            is_object=is_object,
            construct_only=construct_only,
            save_always=save_always,
            default_value=default_value,
            minimum=minimum,
            maximum=maximum,
            version=version,
            deprecated_version=deprecated_version,
            translatable=translatable,
            disable_inline_object=disable_inline_object,
            deprecated=deprecated,
            required=required,
            workspace_default=workspace_default,
            original_owner_id=original_owner_id,
            disabled=disabled,
        )


class CmbSignalInfo(CmbBase):
    __gtype_name__ = "CmbSignalInfo"

    owner_id = GObject.Property(type=str, flags=GObject.ParamFlags.READWRITE | GObject.ParamFlags.CONSTRUCT_ONLY)
    signal_id = GObject.Property(type=str, flags=GObject.ParamFlags.READWRITE | GObject.ParamFlags.CONSTRUCT_ONLY)
    version = GObject.Property(type=str, flags=GObject.ParamFlags.READWRITE | GObject.ParamFlags.CONSTRUCT_ONLY)
    deprecated_version = GObject.Property(type=str, flags=GObject.ParamFlags.READWRITE | GObject.ParamFlags.CONSTRUCT_ONLY)
    detailed = GObject.Property(
        type=bool, flags=GObject.ParamFlags.READWRITE | GObject.ParamFlags.CONSTRUCT_ONLY, default=False
    )

    def __init__(self, **kwargs):
        super().__init__(**kwargs)

    @classmethod
    def from_row(cls, project, owner_id, signal_id, version, deprecated_version, detailed):
        return cls(
            project=project,
            owner_id=owner_id,
            signal_id=signal_id,
            version=version,
            deprecated_version=deprecated_version,
            detailed=detailed,
        )


class CmbBaseTypeInfo(CmbBase):
    __gtype_name__ = "CmbBaseTypeInfo"

    type_id = GObject.Property(type=str, flags=GObject.ParamFlags.READWRITE | GObject.ParamFlags.CONSTRUCT_ONLY)
    parent_id = GObject.Property(type=str, flags=GObject.ParamFlags.READWRITE | GObject.ParamFlags.CONSTRUCT_ONLY)
    library_id = GObject.Property(type=str, flags=GObject.ParamFlags.READWRITE | GObject.ParamFlags.CONSTRUCT_ONLY)
    version = GObject.Property(type=str, flags=GObject.ParamFlags.READWRITE | GObject.ParamFlags.CONSTRUCT_ONLY)
    deprecated_version = GObject.Property(type=str, flags=GObject.ParamFlags.READWRITE | GObject.ParamFlags.CONSTRUCT_ONLY)
    abstract = GObject.Property(
        type=bool, flags=GObject.ParamFlags.READWRITE | GObject.ParamFlags.CONSTRUCT_ONLY, default=False
    )
    derivable = GObject.Property(
        type=bool, flags=GObject.ParamFlags.READWRITE | GObject.ParamFlags.CONSTRUCT_ONLY, default=False
    )
    layout = GObject.Property(type=str, flags=GObject.ParamFlags.READWRITE | GObject.ParamFlags.CONSTRUCT_ONLY)
    category = GObject.Property(type=str, flags=GObject.ParamFlags.READWRITE | GObject.ParamFlags.CONSTRUCT_ONLY)
    workspace_type = GObject.Property(type=str, flags=GObject.ParamFlags.READWRITE | GObject.ParamFlags.CONSTRUCT_ONLY)

    def __init__(self, **kwargs):
        super().__init__(**kwargs)

    @classmethod
    def from_row(
        cls,
        project,
        type_id,
        parent_id,
        library_id,
        version,
        deprecated_version,
        abstract,
        derivable,
        layout,
        category,
        workspace_type,
    ):
        return cls(
            project=project,
            type_id=type_id,
            parent_id=parent_id,
            library_id=library_id,
            version=version,
            deprecated_version=deprecated_version,
            abstract=abstract,
            derivable=derivable,
            layout=layout,
            category=category,
            workspace_type=workspace_type,
        )


class CmbBaseTypeDataInfo(CmbBase):
    __gtype_name__ = "CmbBaseTypeDataInfo"

    owner_id = GObject.Property(type=str, flags=GObject.ParamFlags.READWRITE | GObject.ParamFlags.CONSTRUCT_ONLY)
    data_id = GObject.Property(type=int, flags=GObject.ParamFlags.READWRITE | GObject.ParamFlags.CONSTRUCT_ONLY)
    parent_id = GObject.Property(type=int, flags=GObject.ParamFlags.READWRITE | GObject.ParamFlags.CONSTRUCT_ONLY)
    key = GObject.Property(type=str, flags=GObject.ParamFlags.READWRITE | GObject.ParamFlags.CONSTRUCT_ONLY)
    type_id = GObject.Property(type=str, flags=GObject.ParamFlags.READWRITE | GObject.ParamFlags.CONSTRUCT_ONLY)
    translatable = GObject.Property(
        type=bool, flags=GObject.ParamFlags.READWRITE | GObject.ParamFlags.CONSTRUCT_ONLY, default=False
    )

    def __init__(self, **kwargs):
        super().__init__(**kwargs)

    @classmethod
    def from_row(cls, project, owner_id, data_id, parent_id, key, type_id, translatable):
        return cls(
            project=project,
            owner_id=owner_id,
            data_id=data_id,
            parent_id=parent_id,
            key=key,
            type_id=type_id,
            translatable=translatable,
        )


class CmbBaseTypeDataArgInfo(CmbBase):
    __gtype_name__ = "CmbBaseTypeDataArgInfo"

    owner_id = GObject.Property(type=str, flags=GObject.ParamFlags.READWRITE | GObject.ParamFlags.CONSTRUCT_ONLY)
    data_id = GObject.Property(type=int, flags=GObject.ParamFlags.READWRITE | GObject.ParamFlags.CONSTRUCT_ONLY)
    key = GObject.Property(type=str, flags=GObject.ParamFlags.READWRITE | GObject.ParamFlags.CONSTRUCT_ONLY)
    type_id = GObject.Property(type=str, flags=GObject.ParamFlags.READWRITE | GObject.ParamFlags.CONSTRUCT_ONLY)

    def __init__(self, **kwargs):
        super().__init__(**kwargs)

    @classmethod
    def from_row(cls, project, owner_id, data_id, key, type_id):
        return cls(project=project, owner_id=owner_id, data_id=data_id, key=key, type_id=type_id)


class CmbTypeChildInfo(CmbBase):
    __gtype_name__ = "CmbTypeChildInfo"

    type_id = GObject.Property(type=str, flags=GObject.ParamFlags.READWRITE | GObject.ParamFlags.CONSTRUCT_ONLY)
    child_type = GObject.Property(type=str, flags=GObject.ParamFlags.READWRITE | GObject.ParamFlags.CONSTRUCT_ONLY)
    max_children = GObject.Property(type=int, flags=GObject.ParamFlags.READWRITE | GObject.ParamFlags.CONSTRUCT_ONLY)
    linked_property_id = GObject.Property(type=str, flags=GObject.ParamFlags.READWRITE | GObject.ParamFlags.CONSTRUCT_ONLY)

    def __init__(self, **kwargs):
        super().__init__(**kwargs)

    @classmethod
    def from_row(cls, project, type_id, child_type, max_children, linked_property_id):
        return cls(
            project=project,
            type_id=type_id,
            child_type=child_type,
            max_children=max_children,
            linked_property_id=linked_property_id,
        )


class CmbBaseTypeInternalChildInfo(CmbBase):
    __gtype_name__ = "CmbBaseTypeInternalChildInfo"

    type_id = GObject.Property(type=str, flags=GObject.ParamFlags.READWRITE | GObject.ParamFlags.CONSTRUCT_ONLY)
    internal_child_id = GObject.Property(type=str, flags=GObject.ParamFlags.READWRITE | GObject.ParamFlags.CONSTRUCT_ONLY)
    internal_parent_id = GObject.Property(type=str, flags=GObject.ParamFlags.READWRITE | GObject.ParamFlags.CONSTRUCT_ONLY)
    internal_type = GObject.Property(type=str, flags=GObject.ParamFlags.READWRITE | GObject.ParamFlags.CONSTRUCT_ONLY)
    creation_property_id = GObject.Property(type=str, flags=GObject.ParamFlags.READWRITE | GObject.ParamFlags.CONSTRUCT_ONLY)

    def __init__(self, **kwargs):
        super().__init__(**kwargs)

    @classmethod
    def from_row(cls, project, type_id, internal_child_id, internal_parent_id, internal_type, creation_property_id):
        return cls(
            project=project,
            type_id=type_id,
            internal_child_id=internal_child_id,
            internal_parent_id=internal_parent_id,
            internal_type=internal_type,
            creation_property_id=creation_property_id,
        )


class CmbBaseUI(CmbBaseFileMonitor):
    __gtype_name__ = "CmbBaseUI"

    ui_id = GObject.Property(type=int, flags=GObject.ParamFlags.READWRITE | GObject.ParamFlags.CONSTRUCT_ONLY)

    def __init__(self, **kwargs):
        super().__init__(**kwargs)

    @classmethod
    def from_row(
        cls,
        project,
        ui_id,
        template_id,
        name,
        filename,
        description,
        copyright,
        authors,
        license_id,
        translation_domain,
        comment,
        custom_fragment,
    ):
        return cls(project=project, ui_id=ui_id)

    @GObject.Property(type=int)
    def template_id(self):
        return self.db_get("SELECT template_id FROM ui WHERE (ui_id) IS (?);", (self.ui_id,))

    @template_id.setter
    def _set_template_id(self, value):
        self.db_set("UPDATE ui SET template_id=? WHERE (ui_id) IS (?);", (self.ui_id,), value)

    @GObject.Property(type=str)
    def name(self):
        return self.db_get("SELECT name FROM ui WHERE (ui_id) IS (?);", (self.ui_id,))

    @name.setter
    def _set_name(self, value):
        self.db_set("UPDATE ui SET name=? WHERE (ui_id) IS (?);", (self.ui_id,), value)

    @GObject.Property(type=str)
    def filename(self):
        return self.db_get("SELECT filename FROM ui WHERE (ui_id) IS (?);", (self.ui_id,))

    @filename.setter
    def _set_filename(self, value):
        self.db_set("UPDATE ui SET filename=? WHERE (ui_id) IS (?);", (self.ui_id,), value)

    @GObject.Property(type=str)
    def description(self):
        return self.db_get("SELECT description FROM ui WHERE (ui_id) IS (?);", (self.ui_id,))

    @description.setter
    def _set_description(self, value):
        self.db_set("UPDATE ui SET description=? WHERE (ui_id) IS (?);", (self.ui_id,), value)

    @GObject.Property(type=str)
    def copyright(self):
        return self.db_get("SELECT copyright FROM ui WHERE (ui_id) IS (?);", (self.ui_id,))

    @copyright.setter
    def _set_copyright(self, value):
        self.db_set("UPDATE ui SET copyright=? WHERE (ui_id) IS (?);", (self.ui_id,), value)

    @GObject.Property(type=str)
    def authors(self):
        return self.db_get("SELECT authors FROM ui WHERE (ui_id) IS (?);", (self.ui_id,))

    @authors.setter
    def _set_authors(self, value):
        self.db_set("UPDATE ui SET authors=? WHERE (ui_id) IS (?);", (self.ui_id,), value)

    @GObject.Property(type=str)
    def license_id(self):
        return self.db_get("SELECT license_id FROM ui WHERE (ui_id) IS (?);", (self.ui_id,))

    @license_id.setter
    def _set_license_id(self, value):
        self.db_set("UPDATE ui SET license_id=? WHERE (ui_id) IS (?);", (self.ui_id,), value)

    @GObject.Property(type=str)
    def translation_domain(self):
        return self.db_get("SELECT translation_domain FROM ui WHERE (ui_id) IS (?);", (self.ui_id,))

    @translation_domain.setter
    def _set_translation_domain(self, value):
        self.db_set("UPDATE ui SET translation_domain=? WHERE (ui_id) IS (?);", (self.ui_id,), value)

    @GObject.Property(type=str)
    def comment(self):
        return self.db_get("SELECT comment FROM ui WHERE (ui_id) IS (?);", (self.ui_id,))

    @comment.setter
    def _set_comment(self, value):
        self.db_set("UPDATE ui SET comment=? WHERE (ui_id) IS (?);", (self.ui_id,), value)

    @GObject.Property(type=str)
    def custom_fragment(self):
        return self.db_get("SELECT custom_fragment FROM ui WHERE (ui_id) IS (?);", (self.ui_id,))

    @custom_fragment.setter
    def _set_custom_fragment(self, value):
        self.db_set("UPDATE ui SET custom_fragment=? WHERE (ui_id) IS (?);", (self.ui_id,), value)


class CmbBaseCSS(CmbBaseFileMonitor):
    __gtype_name__ = "CmbBaseCSS"

    css_id = GObject.Property(type=int, flags=GObject.ParamFlags.READWRITE | GObject.ParamFlags.CONSTRUCT_ONLY)

    def __init__(self, **kwargs):
        super().__init__(**kwargs)

    @classmethod
    def from_row(cls, project, css_id, filename, css, priority, is_global):
        return cls(project=project, css_id=css_id)

    @GObject.Property(type=str)
    def filename(self):
        return self.db_get("SELECT filename FROM css WHERE (css_id) IS (?);", (self.css_id,))

    @filename.setter
    def _set_filename(self, value):
        self.db_set("UPDATE css SET filename=? WHERE (css_id) IS (?);", (self.css_id,), value)

    @GObject.Property(type=str)
    def css(self):
        return self.db_get("SELECT css FROM css WHERE (css_id) IS (?);", (self.css_id,))

    @css.setter
    def _set_css(self, value):
        self.db_set("UPDATE css SET css=? WHERE (css_id) IS (?);", (self.css_id,), value)

    @GObject.Property(type=int)
    def priority(self):
        return self.db_get("SELECT priority FROM css WHERE (css_id) IS (?);", (self.css_id,))

    @priority.setter
    def _set_priority(self, value):
        self.db_set("UPDATE css SET priority=? WHERE (css_id) IS (?);", (self.css_id,), value)

    @GObject.Property(type=bool, default=False)
    def is_global(self):
        return self.db_get("SELECT is_global FROM css WHERE (css_id) IS (?);", (self.css_id,))

    @is_global.setter
    def _set_is_global(self, value):
        self.db_set("UPDATE css SET is_global=? WHERE (css_id) IS (?);", (self.css_id,), value)


class CmbBaseGResource(CmbBaseFileMonitor):
    __gtype_name__ = "CmbBaseGResource"

    gresource_id = GObject.Property(type=int, flags=GObject.ParamFlags.READWRITE | GObject.ParamFlags.CONSTRUCT_ONLY)
    resource_type = GObject.Property(type=str, flags=GObject.ParamFlags.READWRITE | GObject.ParamFlags.CONSTRUCT_ONLY)

    def __init__(self, **kwargs):
        super().__init__(**kwargs)

    @classmethod
    def from_row(
        cls,
        project,
        gresource_id,
        resource_type,
        parent_id,
        position,
        gresources_filename,
        gresource_prefix,
        file_filename,
        file_compressed,
        file_preprocess,
        file_alias,
    ):
        return cls(project=project, gresource_id=gresource_id)

    @GObject.Property(type=int)
    def parent_id(self):
        return self.db_get("SELECT parent_id FROM gresource WHERE (gresource_id) IS (?);", (self.gresource_id,))

    @parent_id.setter
    def _set_parent_id(self, value):
        self.db_set("UPDATE gresource SET parent_id=? WHERE (gresource_id) IS (?);", (self.gresource_id,), value)

    @GObject.Property(type=int)
    def position(self):
        return self.db_get("SELECT position FROM gresource WHERE (gresource_id) IS (?);", (self.gresource_id,))

    @position.setter
    def _set_position(self, value):
        self.db_set("UPDATE gresource SET position=? WHERE (gresource_id) IS (?);", (self.gresource_id,), value)

    @GObject.Property(type=str)
    def gresources_filename(self):
        return self.db_get("SELECT gresources_filename FROM gresource WHERE (gresource_id) IS (?);", (self.gresource_id,))

    @gresources_filename.setter
    def _set_gresources_filename(self, value):
        self.db_set("UPDATE gresource SET gresources_filename=? WHERE (gresource_id) IS (?);", (self.gresource_id,), value)

    @GObject.Property(type=str)
    def gresource_prefix(self):
        return self.db_get("SELECT gresource_prefix FROM gresource WHERE (gresource_id) IS (?);", (self.gresource_id,))

    @gresource_prefix.setter
    def _set_gresource_prefix(self, value):
        self.db_set("UPDATE gresource SET gresource_prefix=? WHERE (gresource_id) IS (?);", (self.gresource_id,), value)

    @GObject.Property(type=str)
    def file_filename(self):
        return self.db_get("SELECT file_filename FROM gresource WHERE (gresource_id) IS (?);", (self.gresource_id,))

    @file_filename.setter
    def _set_file_filename(self, value):
        self.db_set("UPDATE gresource SET file_filename=? WHERE (gresource_id) IS (?);", (self.gresource_id,), value)

    @GObject.Property(type=bool, default=False)
    def file_compressed(self):
        return self.db_get("SELECT file_compressed FROM gresource WHERE (gresource_id) IS (?);", (self.gresource_id,))

    @file_compressed.setter
    def _set_file_compressed(self, value):
        self.db_set("UPDATE gresource SET file_compressed=? WHERE (gresource_id) IS (?);", (self.gresource_id,), value)

    @GObject.Property(type=str)
    def file_preprocess(self):
        return self.db_get("SELECT file_preprocess FROM gresource WHERE (gresource_id) IS (?);", (self.gresource_id,))

    @file_preprocess.setter
    def _set_file_preprocess(self, value):
        self.db_set("UPDATE gresource SET file_preprocess=? WHERE (gresource_id) IS (?);", (self.gresource_id,), value)

    @GObject.Property(type=str)
    def file_alias(self):
        return self.db_get("SELECT file_alias FROM gresource WHERE (gresource_id) IS (?);", (self.gresource_id,))

    @file_alias.setter
    def _set_file_alias(self, value):
        self.db_set("UPDATE gresource SET file_alias=? WHERE (gresource_id) IS (?);", (self.gresource_id,), value)


class CmbBaseProperty(CmbBase):
    __gtype_name__ = "CmbBaseProperty"

    ui_id = GObject.Property(type=int, flags=GObject.ParamFlags.READWRITE | GObject.ParamFlags.CONSTRUCT_ONLY)
    object_id = GObject.Property(type=int, flags=GObject.ParamFlags.READWRITE | GObject.ParamFlags.CONSTRUCT_ONLY)
    owner_id = GObject.Property(type=str, flags=GObject.ParamFlags.READWRITE | GObject.ParamFlags.CONSTRUCT_ONLY)
    property_id = GObject.Property(type=str, flags=GObject.ParamFlags.READWRITE | GObject.ParamFlags.CONSTRUCT_ONLY)

    def __init__(self, **kwargs):
        super().__init__(**kwargs)

    @classmethod
    def from_row(
        cls,
        project,
        ui_id,
        object_id,
        owner_id,
        property_id,
        value,
        translatable,
        comment,
        translation_context,
        translation_comments,
        inline_object_id,
        bind_source_id,
        bind_owner_id,
        bind_property_id,
        bind_flags,
        binding_expression_id,
        binding_expression_object_id,
        serialize_default_value,
    ):
        return cls(project=project, ui_id=ui_id, object_id=object_id, owner_id=owner_id, property_id=property_id)

    @GObject.Property(type=str)
    def value(self):
        return self.db_get(
            "SELECT value FROM object_property WHERE (ui_id, object_id, owner_id, property_id) IS (?, ?, ?, ?);",
            (
                self.ui_id,
                self.object_id,
                self.owner_id,
                self.property_id,
            ),
        )

    @value.setter
    def _set_value(self, value):
        self.db_set(
            "UPDATE object_property SET value=? WHERE (ui_id, object_id, owner_id, property_id) IS (?, ?, ?, ?);",
            (
                self.ui_id,
                self.object_id,
                self.owner_id,
                self.property_id,
            ),
            value,
        )

    @GObject.Property(type=bool, default=False)
    def translatable(self):
        return self.db_get(
            "SELECT translatable FROM object_property WHERE (ui_id, object_id, owner_id, property_id) IS (?, ?, ?, ?);",
            (
                self.ui_id,
                self.object_id,
                self.owner_id,
                self.property_id,
            ),
        )

    @translatable.setter
    def _set_translatable(self, value):
        self.db_set(
            "UPDATE object_property SET translatable=? WHERE (ui_id, object_id, owner_id, property_id) IS (?, ?, ?, ?);",
            (
                self.ui_id,
                self.object_id,
                self.owner_id,
                self.property_id,
            ),
            value,
        )

    @GObject.Property(type=str)
    def comment(self):
        return self.db_get(
            "SELECT comment FROM object_property WHERE (ui_id, object_id, owner_id, property_id) IS (?, ?, ?, ?);",
            (
                self.ui_id,
                self.object_id,
                self.owner_id,
                self.property_id,
            ),
        )

    @comment.setter
    def _set_comment(self, value):
        self.db_set(
            "UPDATE object_property SET comment=? WHERE (ui_id, object_id, owner_id, property_id) IS (?, ?, ?, ?);",
            (
                self.ui_id,
                self.object_id,
                self.owner_id,
                self.property_id,
            ),
            value,
        )

    @GObject.Property(type=str)
    def translation_context(self):
        return self.db_get(
            "SELECT translation_context FROM object_property WHERE (ui_id, object_id, owner_id, property_id) IS (?, ?, ?, ?);",
            (
                self.ui_id,
                self.object_id,
                self.owner_id,
                self.property_id,
            ),
        )

    @translation_context.setter
    def _set_translation_context(self, value):
        self.db_set(
            "UPDATE object_property SET translation_context=? WHERE (ui_id, object_id, owner_id, property_id) IS (?, ?, ?, ?);",
            (
                self.ui_id,
                self.object_id,
                self.owner_id,
                self.property_id,
            ),
            value,
        )

    @GObject.Property(type=str)
    def translation_comments(self):
        return self.db_get(
            "SELECT translation_comments FROM object_property WHERE (ui_id, object_id, owner_id, property_id) IS (?, ?, ?, ?);",
            (
                self.ui_id,
                self.object_id,
                self.owner_id,
                self.property_id,
            ),
        )

    @translation_comments.setter
    def _set_translation_comments(self, value):
        self.db_set(
            "UPDATE object_property SET translation_comments=? WHERE (ui_id, object_id, owner_id, property_id) IS (?, ?, ?, ?);",
            (
                self.ui_id,
                self.object_id,
                self.owner_id,
                self.property_id,
            ),
            value,
        )

    @GObject.Property(type=int)
    def inline_object_id(self):
        return self.db_get(
            "SELECT inline_object_id FROM object_property WHERE (ui_id, object_id, owner_id, property_id) IS (?, ?, ?, ?);",
            (
                self.ui_id,
                self.object_id,
                self.owner_id,
                self.property_id,
            ),
        )

    @inline_object_id.setter
    def _set_inline_object_id(self, value):
        self.db_set(
            "UPDATE object_property SET inline_object_id=? WHERE (ui_id, object_id, owner_id, property_id) IS (?, ?, ?, ?);",
            (
                self.ui_id,
                self.object_id,
                self.owner_id,
                self.property_id,
            ),
            value,
        )

    @GObject.Property(type=int)
    def bind_source_id(self):
        return self.db_get(
            "SELECT bind_source_id FROM object_property WHERE (ui_id, object_id, owner_id, property_id) IS (?, ?, ?, ?);",
            (
                self.ui_id,
                self.object_id,
                self.owner_id,
                self.property_id,
            ),
        )

    @bind_source_id.setter
    def _set_bind_source_id(self, value):
        self.db_set(
            "UPDATE object_property SET bind_source_id=? WHERE (ui_id, object_id, owner_id, property_id) IS (?, ?, ?, ?);",
            (
                self.ui_id,
                self.object_id,
                self.owner_id,
                self.property_id,
            ),
            value,
        )

    @GObject.Property(type=str)
    def bind_owner_id(self):
        return self.db_get(
            "SELECT bind_owner_id FROM object_property WHERE (ui_id, object_id, owner_id, property_id) IS (?, ?, ?, ?);",
            (
                self.ui_id,
                self.object_id,
                self.owner_id,
                self.property_id,
            ),
        )

    @bind_owner_id.setter
    def _set_bind_owner_id(self, value):
        self.db_set(
            "UPDATE object_property SET bind_owner_id=? WHERE (ui_id, object_id, owner_id, property_id) IS (?, ?, ?, ?);",
            (
                self.ui_id,
                self.object_id,
                self.owner_id,
                self.property_id,
            ),
            value,
        )

    @GObject.Property(type=str)
    def bind_property_id(self):
        return self.db_get(
            "SELECT bind_property_id FROM object_property WHERE (ui_id, object_id, owner_id, property_id) IS (?, ?, ?, ?);",
            (
                self.ui_id,
                self.object_id,
                self.owner_id,
                self.property_id,
            ),
        )

    @bind_property_id.setter
    def _set_bind_property_id(self, value):
        self.db_set(
            "UPDATE object_property SET bind_property_id=? WHERE (ui_id, object_id, owner_id, property_id) IS (?, ?, ?, ?);",
            (
                self.ui_id,
                self.object_id,
                self.owner_id,
                self.property_id,
            ),
            value,
        )

    @GObject.Property(type=str)
    def bind_flags(self):
        return self.db_get(
            "SELECT bind_flags FROM object_property WHERE (ui_id, object_id, owner_id, property_id) IS (?, ?, ?, ?);",
            (
                self.ui_id,
                self.object_id,
                self.owner_id,
                self.property_id,
            ),
        )

    @bind_flags.setter
    def _set_bind_flags(self, value):
        self.db_set(
            "UPDATE object_property SET bind_flags=? WHERE (ui_id, object_id, owner_id, property_id) IS (?, ?, ?, ?);",
            (
                self.ui_id,
                self.object_id,
                self.owner_id,
                self.property_id,
            ),
            value,
        )

    @GObject.Property(type=int)
    def binding_expression_id(self):
        return self.db_get(
            "SELECT binding_expression_id FROM object_property WHERE (ui_id, object_id, owner_id, property_id) IS (?, ?, ?, ?);",
            (
                self.ui_id,
                self.object_id,
                self.owner_id,
                self.property_id,
            ),
        )

    @binding_expression_id.setter
    def _set_binding_expression_id(self, value):
        self.db_set(
            "UPDATE object_property SET binding_expression_id=? WHERE (ui_id, object_id, owner_id, property_id) IS (?, ?, ?, ?);",
            (
                self.ui_id,
                self.object_id,
                self.owner_id,
                self.property_id,
            ),
            value,
        )

    @GObject.Property(type=int)
    def binding_expression_object_id(self):
        return self.db_get(
            "SELECT binding_expression_object_id FROM object_property WHERE (ui_id, object_id, owner_id, property_id) IS (?, ?, ?, ?);",
            (
                self.ui_id,
                self.object_id,
                self.owner_id,
                self.property_id,
            ),
        )

    @binding_expression_object_id.setter
    def _set_binding_expression_object_id(self, value):
        self.db_set(
            "UPDATE object_property SET binding_expression_object_id=? WHERE (ui_id, object_id, owner_id, property_id) IS (?, ?, ?, ?);",
            (
                self.ui_id,
                self.object_id,
                self.owner_id,
                self.property_id,
            ),
            value,
        )

    @GObject.Property(type=bool, default=False)
    def serialize_default_value(self):
        return self.db_get(
            "SELECT serialize_default_value FROM object_property WHERE (ui_id, object_id, owner_id, property_id) IS (?, ?, ?, ?);",
            (
                self.ui_id,
                self.object_id,
                self.owner_id,
                self.property_id,
            ),
        )

    @serialize_default_value.setter
    def _set_serialize_default_value(self, value):
        self.db_set(
            "UPDATE object_property SET serialize_default_value=? WHERE (ui_id, object_id, owner_id, property_id) IS (?, ?, ?, ?);",
            (
                self.ui_id,
                self.object_id,
                self.owner_id,
                self.property_id,
            ),
            value,
        )


class CmbBaseLayoutProperty(CmbBase):
    __gtype_name__ = "CmbBaseLayoutProperty"

    ui_id = GObject.Property(type=int, flags=GObject.ParamFlags.READWRITE | GObject.ParamFlags.CONSTRUCT_ONLY)
    object_id = GObject.Property(type=int, flags=GObject.ParamFlags.READWRITE | GObject.ParamFlags.CONSTRUCT_ONLY)
    child_id = GObject.Property(type=int, flags=GObject.ParamFlags.READWRITE | GObject.ParamFlags.CONSTRUCT_ONLY)
    owner_id = GObject.Property(type=str, flags=GObject.ParamFlags.READWRITE | GObject.ParamFlags.CONSTRUCT_ONLY)
    property_id = GObject.Property(type=str, flags=GObject.ParamFlags.READWRITE | GObject.ParamFlags.CONSTRUCT_ONLY)

    def __init__(self, **kwargs):
        super().__init__(**kwargs)

    @classmethod
    def from_row(
        cls,
        project,
        ui_id,
        object_id,
        child_id,
        owner_id,
        property_id,
        value,
        translatable,
        comment,
        translation_context,
        translation_comments,
    ):
        return cls(
            project=project, ui_id=ui_id, object_id=object_id, child_id=child_id, owner_id=owner_id, property_id=property_id
        )

    @GObject.Property(type=str)
    def value(self):
        return self.db_get(
            "SELECT value FROM object_layout_property WHERE (ui_id, object_id, child_id, owner_id, property_id) IS (?, ?, ?, ?, ?);",
            (
                self.ui_id,
                self.object_id,
                self.child_id,
                self.owner_id,
                self.property_id,
            ),
        )

    @value.setter
    def _set_value(self, value):
        self.db_set(
            "UPDATE object_layout_property SET value=? WHERE (ui_id, object_id, child_id, owner_id, property_id) IS (?, ?, ?, ?, ?);",
            (
                self.ui_id,
                self.object_id,
                self.child_id,
                self.owner_id,
                self.property_id,
            ),
            value,
        )

    @GObject.Property(type=bool, default=False)
    def translatable(self):
        return self.db_get(
            "SELECT translatable FROM object_layout_property WHERE (ui_id, object_id, child_id, owner_id, property_id) IS (?, ?, ?, ?, ?);",
            (
                self.ui_id,
                self.object_id,
                self.child_id,
                self.owner_id,
                self.property_id,
            ),
        )

    @translatable.setter
    def _set_translatable(self, value):
        self.db_set(
            "UPDATE object_layout_property SET translatable=? WHERE (ui_id, object_id, child_id, owner_id, property_id) IS (?, ?, ?, ?, ?);",
            (
                self.ui_id,
                self.object_id,
                self.child_id,
                self.owner_id,
                self.property_id,
            ),
            value,
        )

    @GObject.Property(type=str)
    def comment(self):
        return self.db_get(
            "SELECT comment FROM object_layout_property WHERE (ui_id, object_id, child_id, owner_id, property_id) IS (?, ?, ?, ?, ?);",
            (
                self.ui_id,
                self.object_id,
                self.child_id,
                self.owner_id,
                self.property_id,
            ),
        )

    @comment.setter
    def _set_comment(self, value):
        self.db_set(
            "UPDATE object_layout_property SET comment=? WHERE (ui_id, object_id, child_id, owner_id, property_id) IS (?, ?, ?, ?, ?);",
            (
                self.ui_id,
                self.object_id,
                self.child_id,
                self.owner_id,
                self.property_id,
            ),
            value,
        )

    @GObject.Property(type=str)
    def translation_context(self):
        return self.db_get(
            "SELECT translation_context FROM object_layout_property WHERE (ui_id, object_id, child_id, owner_id, property_id) IS (?, ?, ?, ?, ?);",
            (
                self.ui_id,
                self.object_id,
                self.child_id,
                self.owner_id,
                self.property_id,
            ),
        )

    @translation_context.setter
    def _set_translation_context(self, value):
        self.db_set(
            "UPDATE object_layout_property SET translation_context=? WHERE (ui_id, object_id, child_id, owner_id, property_id) IS (?, ?, ?, ?, ?);",
            (
                self.ui_id,
                self.object_id,
                self.child_id,
                self.owner_id,
                self.property_id,
            ),
            value,
        )

    @GObject.Property(type=str)
    def translation_comments(self):
        return self.db_get(
            "SELECT translation_comments FROM object_layout_property WHERE (ui_id, object_id, child_id, owner_id, property_id) IS (?, ?, ?, ?, ?);",
            (
                self.ui_id,
                self.object_id,
                self.child_id,
                self.owner_id,
                self.property_id,
            ),
        )

    @translation_comments.setter
    def _set_translation_comments(self, value):
        self.db_set(
            "UPDATE object_layout_property SET translation_comments=? WHERE (ui_id, object_id, child_id, owner_id, property_id) IS (?, ?, ?, ?, ?);",
            (
                self.ui_id,
                self.object_id,
                self.child_id,
                self.owner_id,
                self.property_id,
            ),
            value,
        )


class CmbSignal(CmbBase):
    __gtype_name__ = "CmbSignal"

    signal_pk = GObject.Property(type=int, flags=GObject.ParamFlags.READWRITE | GObject.ParamFlags.CONSTRUCT_ONLY)

    def __init__(self, **kwargs):
        super().__init__(**kwargs)

    @classmethod
    def from_row(
        cls, project, signal_pk, ui_id, object_id, owner_id, signal_id, handler, detail, user_data, swap, after, comment
    ):
        return cls(project=project, signal_pk=signal_pk)

    @GObject.Property(type=int)
    def ui_id(self):
        return self.db_get("SELECT ui_id FROM object_signal WHERE (signal_pk) IS (?);", (self.signal_pk,))

    @ui_id.setter
    def _set_ui_id(self, value):
        self.db_set("UPDATE object_signal SET ui_id=? WHERE (signal_pk) IS (?);", (self.signal_pk,), value)

    @GObject.Property(type=int)
    def object_id(self):
        return self.db_get("SELECT object_id FROM object_signal WHERE (signal_pk) IS (?);", (self.signal_pk,))

    @object_id.setter
    def _set_object_id(self, value):
        self.db_set("UPDATE object_signal SET object_id=? WHERE (signal_pk) IS (?);", (self.signal_pk,), value)

    @GObject.Property(type=str)
    def owner_id(self):
        return self.db_get("SELECT owner_id FROM object_signal WHERE (signal_pk) IS (?);", (self.signal_pk,))

    @owner_id.setter
    def _set_owner_id(self, value):
        self.db_set("UPDATE object_signal SET owner_id=? WHERE (signal_pk) IS (?);", (self.signal_pk,), value)

    @GObject.Property(type=str)
    def signal_id(self):
        return self.db_get("SELECT signal_id FROM object_signal WHERE (signal_pk) IS (?);", (self.signal_pk,))

    @signal_id.setter
    def _set_signal_id(self, value):
        self.db_set("UPDATE object_signal SET signal_id=? WHERE (signal_pk) IS (?);", (self.signal_pk,), value)

    @GObject.Property(type=str)
    def handler(self):
        return self.db_get("SELECT handler FROM object_signal WHERE (signal_pk) IS (?);", (self.signal_pk,))

    @handler.setter
    def _set_handler(self, value):
        self.db_set("UPDATE object_signal SET handler=? WHERE (signal_pk) IS (?);", (self.signal_pk,), value)

    @GObject.Property(type=str)
    def detail(self):
        return self.db_get("SELECT detail FROM object_signal WHERE (signal_pk) IS (?);", (self.signal_pk,))

    @detail.setter
    def _set_detail(self, value):
        self.db_set("UPDATE object_signal SET detail=? WHERE (signal_pk) IS (?);", (self.signal_pk,), value)

    @GObject.Property(type=int)
    def user_data(self):
        return self.db_get("SELECT user_data FROM object_signal WHERE (signal_pk) IS (?);", (self.signal_pk,))

    @user_data.setter
    def _set_user_data(self, value):
        self.db_set("UPDATE object_signal SET user_data=? WHERE (signal_pk) IS (?);", (self.signal_pk,), value)

    @GObject.Property(type=bool, default=False)
    def swap(self):
        return self.db_get("SELECT swap FROM object_signal WHERE (signal_pk) IS (?);", (self.signal_pk,))

    @swap.setter
    def _set_swap(self, value):
        self.db_set("UPDATE object_signal SET swap=? WHERE (signal_pk) IS (?);", (self.signal_pk,), value)

    @GObject.Property(type=bool, default=False)
    def after(self):
        return self.db_get("SELECT after FROM object_signal WHERE (signal_pk) IS (?);", (self.signal_pk,))

    @after.setter
    def _set_after(self, value):
        self.db_set("UPDATE object_signal SET after=? WHERE (signal_pk) IS (?);", (self.signal_pk,), value)

    @GObject.Property(type=str)
    def comment(self):
        return self.db_get("SELECT comment FROM object_signal WHERE (signal_pk) IS (?);", (self.signal_pk,))

    @comment.setter
    def _set_comment(self, value):
        self.db_set("UPDATE object_signal SET comment=? WHERE (signal_pk) IS (?);", (self.signal_pk,), value)


class CmbBaseObject(CmbBase):
    __gtype_name__ = "CmbBaseObject"

    ui_id = GObject.Property(type=int, flags=GObject.ParamFlags.READWRITE | GObject.ParamFlags.CONSTRUCT_ONLY)
    object_id = GObject.Property(type=int, flags=GObject.ParamFlags.READWRITE | GObject.ParamFlags.CONSTRUCT_ONLY)

    def __init__(self, **kwargs):
        super().__init__(**kwargs)

    @classmethod
    def from_row(
        cls,
        project,
        ui_id,
        object_id,
        type_id,
        name,
        parent_id,
        internal,
        type,
        comment,
        position,
        custom_fragment,
        custom_child_fragment,
    ):
        return cls(project=project, ui_id=ui_id, object_id=object_id)

    @GObject.Property(type=str)
    def type_id(self):
        return self.db_get(
            "SELECT type_id FROM object WHERE (ui_id, object_id) IS (?, ?);",
            (
                self.ui_id,
                self.object_id,
            ),
        )

    @type_id.setter
    def _set_type_id(self, value):
        self.db_set(
            "UPDATE object SET type_id=? WHERE (ui_id, object_id) IS (?, ?);",
            (
                self.ui_id,
                self.object_id,
            ),
            value,
        )

    @GObject.Property(type=str)
    def name(self):
        return self.db_get(
            "SELECT name FROM object WHERE (ui_id, object_id) IS (?, ?);",
            (
                self.ui_id,
                self.object_id,
            ),
        )

    @name.setter
    def _set_name(self, value):
        self.db_set(
            "UPDATE object SET name=? WHERE (ui_id, object_id) IS (?, ?);",
            (
                self.ui_id,
                self.object_id,
            ),
            value,
        )

    @GObject.Property(type=int)
    def parent_id(self):
        return self.db_get(
            "SELECT parent_id FROM object WHERE (ui_id, object_id) IS (?, ?);",
            (
                self.ui_id,
                self.object_id,
            ),
        )

    @parent_id.setter
    def _set_parent_id(self, value):
        self.db_set(
            "UPDATE object SET parent_id=? WHERE (ui_id, object_id) IS (?, ?);",
            (
                self.ui_id,
                self.object_id,
            ),
            value,
        )

    @GObject.Property(type=str)
    def internal(self):
        return self.db_get(
            "SELECT internal FROM object WHERE (ui_id, object_id) IS (?, ?);",
            (
                self.ui_id,
                self.object_id,
            ),
        )

    @internal.setter
    def _set_internal(self, value):
        self.db_set(
            "UPDATE object SET internal=? WHERE (ui_id, object_id) IS (?, ?);",
            (
                self.ui_id,
                self.object_id,
            ),
            value,
        )

    @GObject.Property(type=str)
    def type(self):
        return self.db_get(
            "SELECT type FROM object WHERE (ui_id, object_id) IS (?, ?);",
            (
                self.ui_id,
                self.object_id,
            ),
        )

    @type.setter
    def _set_type(self, value):
        self.db_set(
            "UPDATE object SET type=? WHERE (ui_id, object_id) IS (?, ?);",
            (
                self.ui_id,
                self.object_id,
            ),
            value,
        )

    @GObject.Property(type=str)
    def comment(self):
        return self.db_get(
            "SELECT comment FROM object WHERE (ui_id, object_id) IS (?, ?);",
            (
                self.ui_id,
                self.object_id,
            ),
        )

    @comment.setter
    def _set_comment(self, value):
        self.db_set(
            "UPDATE object SET comment=? WHERE (ui_id, object_id) IS (?, ?);",
            (
                self.ui_id,
                self.object_id,
            ),
            value,
        )

    @GObject.Property(type=int)
    def position(self):
        return self.db_get(
            "SELECT position FROM object WHERE (ui_id, object_id) IS (?, ?);",
            (
                self.ui_id,
                self.object_id,
            ),
        )

    @position.setter
    def _set_position(self, value):
        self.db_set(
            "UPDATE object SET position=? WHERE (ui_id, object_id) IS (?, ?);",
            (
                self.ui_id,
                self.object_id,
            ),
            value,
        )

    @GObject.Property(type=str)
    def custom_fragment(self):
        return self.db_get(
            "SELECT custom_fragment FROM object WHERE (ui_id, object_id) IS (?, ?);",
            (
                self.ui_id,
                self.object_id,
            ),
        )

    @custom_fragment.setter
    def _set_custom_fragment(self, value):
        self.db_set(
            "UPDATE object SET custom_fragment=? WHERE (ui_id, object_id) IS (?, ?);",
            (
                self.ui_id,
                self.object_id,
            ),
            value,
        )

    @GObject.Property(type=str)
    def custom_child_fragment(self):
        return self.db_get(
            "SELECT custom_child_fragment FROM object WHERE (ui_id, object_id) IS (?, ?);",
            (
                self.ui_id,
                self.object_id,
            ),
        )

    @custom_child_fragment.setter
    def _set_custom_child_fragment(self, value):
        self.db_set(
            "UPDATE object SET custom_child_fragment=? WHERE (ui_id, object_id) IS (?, ?);",
            (
                self.ui_id,
                self.object_id,
            ),
            value,
        )


class CmbBaseObjectData(CmbBase):
    __gtype_name__ = "CmbBaseObjectData"

    ui_id = GObject.Property(type=int, flags=GObject.ParamFlags.READWRITE | GObject.ParamFlags.CONSTRUCT_ONLY)
    object_id = GObject.Property(type=int, flags=GObject.ParamFlags.READWRITE | GObject.ParamFlags.CONSTRUCT_ONLY)
    owner_id = GObject.Property(type=str, flags=GObject.ParamFlags.READWRITE | GObject.ParamFlags.CONSTRUCT_ONLY)
    data_id = GObject.Property(type=int, flags=GObject.ParamFlags.READWRITE | GObject.ParamFlags.CONSTRUCT_ONLY)
    id = GObject.Property(type=int, flags=GObject.ParamFlags.READWRITE | GObject.ParamFlags.CONSTRUCT_ONLY)

    def __init__(self, **kwargs):
        super().__init__(**kwargs)

    @classmethod
    def from_row(
        cls,
        project,
        ui_id,
        object_id,
        owner_id,
        data_id,
        id,
        value,
        parent_id,
        comment,
        translatable,
        translation_context,
        translation_comments,
    ):
        return cls(project=project, ui_id=ui_id, object_id=object_id, owner_id=owner_id, data_id=data_id, id=id)

    @GObject.Property(type=str)
    def value(self):
        return self.db_get(
            "SELECT value FROM object_data WHERE (ui_id, object_id, owner_id, data_id, id) IS (?, ?, ?, ?, ?);",
            (
                self.ui_id,
                self.object_id,
                self.owner_id,
                self.data_id,
                self.id,
            ),
        )

    @value.setter
    def _set_value(self, value):
        self.db_set(
            "UPDATE object_data SET value=? WHERE (ui_id, object_id, owner_id, data_id, id) IS (?, ?, ?, ?, ?);",
            (
                self.ui_id,
                self.object_id,
                self.owner_id,
                self.data_id,
                self.id,
            ),
            value,
        )

    @GObject.Property(type=int)
    def parent_id(self):
        return self.db_get(
            "SELECT parent_id FROM object_data WHERE (ui_id, object_id, owner_id, data_id, id) IS (?, ?, ?, ?, ?);",
            (
                self.ui_id,
                self.object_id,
                self.owner_id,
                self.data_id,
                self.id,
            ),
        )

    @parent_id.setter
    def _set_parent_id(self, value):
        self.db_set(
            "UPDATE object_data SET parent_id=? WHERE (ui_id, object_id, owner_id, data_id, id) IS (?, ?, ?, ?, ?);",
            (
                self.ui_id,
                self.object_id,
                self.owner_id,
                self.data_id,
                self.id,
            ),
            value,
        )

    @GObject.Property(type=str)
    def comment(self):
        return self.db_get(
            "SELECT comment FROM object_data WHERE (ui_id, object_id, owner_id, data_id, id) IS (?, ?, ?, ?, ?);",
            (
                self.ui_id,
                self.object_id,
                self.owner_id,
                self.data_id,
                self.id,
            ),
        )

    @comment.setter
    def _set_comment(self, value):
        self.db_set(
            "UPDATE object_data SET comment=? WHERE (ui_id, object_id, owner_id, data_id, id) IS (?, ?, ?, ?, ?);",
            (
                self.ui_id,
                self.object_id,
                self.owner_id,
                self.data_id,
                self.id,
            ),
            value,
        )

    @GObject.Property(type=bool, default=False)
    def translatable(self):
        return self.db_get(
            "SELECT translatable FROM object_data WHERE (ui_id, object_id, owner_id, data_id, id) IS (?, ?, ?, ?, ?);",
            (
                self.ui_id,
                self.object_id,
                self.owner_id,
                self.data_id,
                self.id,
            ),
        )

    @translatable.setter
    def _set_translatable(self, value):
        self.db_set(
            "UPDATE object_data SET translatable=? WHERE (ui_id, object_id, owner_id, data_id, id) IS (?, ?, ?, ?, ?);",
            (
                self.ui_id,
                self.object_id,
                self.owner_id,
                self.data_id,
                self.id,
            ),
            value,
        )

    @GObject.Property(type=str)
    def translation_context(self):
        return self.db_get(
            "SELECT translation_context FROM object_data WHERE (ui_id, object_id, owner_id, data_id, id) IS (?, ?, ?, ?, ?);",
            (
                self.ui_id,
                self.object_id,
                self.owner_id,
                self.data_id,
                self.id,
            ),
        )

    @translation_context.setter
    def _set_translation_context(self, value):
        self.db_set(
            "UPDATE object_data SET translation_context=? WHERE (ui_id, object_id, owner_id, data_id, id) IS (?, ?, ?, ?, ?);",
            (
                self.ui_id,
                self.object_id,
                self.owner_id,
                self.data_id,
                self.id,
            ),
            value,
        )

    @GObject.Property(type=str)
    def translation_comments(self):
        return self.db_get(
            "SELECT translation_comments FROM object_data WHERE (ui_id, object_id, owner_id, data_id, id) IS (?, ?, ?, ?, ?);",
            (
                self.ui_id,
                self.object_id,
                self.owner_id,
                self.data_id,
                self.id,
            ),
        )

    @translation_comments.setter
    def _set_translation_comments(self, value):
        self.db_set(
            "UPDATE object_data SET translation_comments=? WHERE (ui_id, object_id, owner_id, data_id, id) IS (?, ?, ?, ?, ?);",
            (
                self.ui_id,
                self.object_id,
                self.owner_id,
                self.data_id,
                self.id,
            ),
            value,
        )



#
# CmbCSSEditor - Cambalache CSS Editor
#
# Copyright (C) 2022-2024  Juan Pablo Ugarte
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation;
# version 2.1 of the License.
#
# library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
# Authors:
#   Juan Pablo Ugarte <juanpablougarte@gmail.com>
#
# SPDX-License-Identifier: LGPL-2.1-only
#

from gi.repository import GObject, Gio, Gtk, GtkSource, Pango
from cambalache import utils, _
from .cmb_css import CmbCSS


class CmbCSSPriority(GObject.Object):
    __gtype_name__ = "CmbCSSPriority"

    value = GObject.Property(type=int, flags=GObject.ParamFlags.READWRITE)
    name = GObject.Property(type=str, flags=GObject.ParamFlags.READWRITE)


class CmbCSSHoverProvider(GObject.Object, GtkSource.HoverProvider):
    __gtype_name__ = "CmbCSSHoverProvider"

    def __init__(self, **kwargs):
        super().__init__(**kwargs)

    def do_populate(self, context, display):
        valid, pos = context.get_iter()
        if not valid:
            return False

        # Get a list of error tags using parsing_error attribute
        if errors := [tag.parsing_error for tag in pos.get_tags() if hasattr(tag, "parsing_error")]:
            display.append(Gtk.Label(label="\n".join(errors)))
            display.props.visible = True
            return True

        # FIXME: returning False gives a warning
        # (run-dev.py:167985): GLib-GIO-CRITICAL **: 09:28:07.517: g_task_return_error: assertion 'error != NULL' failed
        # (run-dev.py:167985): GLib-GIO-CRITICAL **: 09:28:07.517: GTask gtk_source_hover_provider_real_populate_async
        #   (source object: 0x1d15adf0, source tag: 0x7ff2c684c060) finalized without ever returning (using g_task_return_*()).
        #   This potentially indicates a bug in the program.
        # As a workaround hide display and return true
        display.props.visible = False
        return True


@Gtk.Template(resource_path="/ar/xjuan/Cambalache/cmb_css_editor.ui")
class CmbCSSEditor(Gtk.Box):
    __gtype_name__ = "CmbCSSEditor"

    filename = Gtk.Template.Child()
    priority = Gtk.Template.Child()
    priority_dropdown = Gtk.Template.Child()
    is_global = Gtk.Template.Child()

    ui_menu_button = Gtk.Template.Child()
    ui_box = Gtk.Template.Child()
    view = Gtk.Template.Child()

    fields = [("filename", "cmb-value"), ("priority", "value"), ("is_global", "active")]

    def __init__(self, **kwargs):
        self._object = None
        self._bindings = []
        self.__tags = []

        super().__init__(**kwargs)

        self.priority.set_range(0, 10000)
        self.priority.set_increments(10, 100)

        self.priority_list = Gio.ListStore(item_type=CmbCSSPriority)

        for name, value in [
            # Translators: GTK_STYLE_PROVIDER_PRIORITY_FALLBACK
            (_("Fallback (1)"), 1),
            # Translators: GTK_STYLE_PROVIDER_PRIORITY_THEME
            (_("Theme (200)"), 200),
            # Translators: GTK_STYLE_PROVIDER_PRIORITY_SETTINGS
            (_("Settings (400)"), 400),
            # Translators: GTK_STYLE_PROVIDER_PRIORITY_APPLICATION
            (_("Application (600)"), 600),
            # Translators: GTK_STYLE_PROVIDER_PRIORITY_USER
            (_("User (800)"), 800),
            # Translators: label for custom GtkStyleProvider priority
            (_("Custom"), -1),
        ]:
            self.priority_list.append(CmbCSSPriority(name=name, value=value))

        self.priority_dropdown.props.model = self.priority_list
        self.__expression = Gtk.PropertyExpression.new(CmbCSSPriority, None, "name")
        self.priority_dropdown.props.expression = self.__expression

        # Add provider for parsing error tooltip
        hover = self.view.get_hover()
        self.__hover_provider = CmbCSSHoverProvider()
        hover.add_provider(self.__hover_provider)

    @GObject.Property(type=CmbCSS)
    def object(self):
        return self._object

    @object.setter
    def _set_object(self, obj):
        if obj == self._object:
            return

        for binding in self._bindings:
            binding.unbind()

        if self._object:
            self._object.project.disconnect_by_func(self.__on_ui_added_removed)
            self._object.disconnect_by_func(self.__on_provider_for_notify)

        self._bindings = []

        self._object = obj

        if obj is None:
            self.set_sensitive(False)
            return

        self.filename.dirname = obj.project.dirname
        self.set_sensitive(True)

        for field, target in self.fields:
            binding = GObject.Object.bind_property(
                obj,
                field,
                getattr(self, field),
                target,
                GObject.BindingFlags.SYNC_CREATE | GObject.BindingFlags.BIDIRECTIONAL,
            )
            self._bindings.append(binding)

        binding = GObject.Object.bind_property(
            obj, "css", self.view, "text", GObject.BindingFlags.SYNC_CREATE | GObject.BindingFlags.BIDIRECTIONAL
        )
        self._bindings.append(binding)

        def transform_to(b, value):
            return {
                1: 0,
                200: 1,
                400: 2,
                600: 3,
                800: 4,
            }.get(value, 5)

        def transform_from(b, value):

            self.priority.props.visible = value == 5

            return {
                0: 1,
                1: 200,
                2: 400,
                3: 600,
                4: 800,
            }.get(value, self.priority.props.value)

        binding = GObject.Object.bind_property(
            self.priority,
            "value",
            self.priority_dropdown,
            "selected",
            GObject.BindingFlags.SYNC_CREATE | GObject.BindingFlags.BIDIRECTIONAL,
            transform_to=transform_to,
            transform_from=transform_from
        )
        self._bindings.append(binding)
        self.priority.props.visible = transform_to(binding, obj.priority) == 5

        obj.project.connect("ui-added", self.__on_ui_added_removed)
        obj.project.connect("ui-removed", self.__on_ui_added_removed)
        obj.connect("notify::provider-for", self.__on_provider_for_notify)
        obj.connect("parsing-status", self.__on_parsing_status)

        self.__update_provider_for()
        self.__update_ui_button_label()

    def __update_provider_for(self):
        # Remove all css_ui check buttons
        for child in utils.widget_get_children(self.ui_box):
            self.ui_box.remove(child)

        if self._object is None:
            return

        ui_list = self._object.project.get_ui_list()
        provider_for = self._object.provider_for

        # Generate a check button for each UI
        for ui in ui_list:
            check = Gtk.CheckButton(
                label=ui.display_name, active=ui.ui_id in provider_for, halign=Gtk.Align.START, visible=True
            )
            check.connect("toggled", self.__on_check_button_toggled, ui)
            self.ui_box.append(check)

    def __on_check_button_toggled(self, button, ui):
        if button.props.active:
            self.object.add_ui(ui)
        else:
            self.object.remove_ui(ui)

        self.__update_ui_button_label()

    def __update_ui_button_label(self):
        n = 0
        first_one = None
        child = self.ui_box.get_first_child()

        while child is not None:
            if child.props.active:
                n += 1

                if first_one is None:
                    first_one = child

            child = child.get_next_sibling()

        if first_one is None:
            self.ui_menu_button.props.label = _("None")
        else:
            self.ui_menu_button.props.label = f"{first_one.props.label} + {n - 1}" if n > 1 else first_one.props.label

    def __on_ui_added_removed(self, project, ui):
        self.__update_provider_for()

    def __on_provider_for_notify(self, obj, pspec):
        self.__update_provider_for()

    def __on_parsing_status(self, obj, error_count, message, start, end):
        buffer = self.view.buffer

        # Clear tags if there is no error or is the first error
        if error_count <= 1:
            buffer_start, buffer_end = buffer.get_bounds()

            # Remove all error tags
            for tag in self.__tags:
                buffer.remove_tag(tag, buffer_start, buffer_end)

            self.__tags = []

        if error_count and message:
            tag = buffer.create_tag(None, underline=Pango.Underline.ERROR)

            # Set error message in parsing_error attribute
            setattr(tag, "parsing_error", message)

            # Apply tag in buffer
            buffer.apply_tag(tag, buffer.get_iter_at_offset(start), buffer.get_iter_at_offset(end))

            # Keep a list of error tags to delete
            self.__tags.append(tag)

    @GObject.Signal(
        flags=GObject.SignalFlags.RUN_LAST,
        return_type=bool,
        arg_types=(),
        accumulator=GObject.signal_accumulator_true_handled,
    )
    def remove_css(self):
        if self.object:
            self.object.project.remove_css(self.object)

        return True


Gtk.WidgetClass.set_css_name(CmbCSSEditor, "CmbCSSEditor")



#
# CmbNotificationListView
#
# Copyright (C) 2025  Juan Pablo Ugarte
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation;
# version 2.1 of the License.
#
# library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
# Authors:
#   Juan Pablo Ugarte <juanpablougarte@gmail.com>
#
# SPDX-License-Identifier: LGPL-2.1-only
#

from cambalache import getLogger
from gi.repository import GObject, Gtk
from .cmb_version_notification_view import CmbVersionNotificationView
from .cmb_message_notification_view import CmbMessageNotificationView
from .cmb_poll_notification_view import CmbPollNotificationView
from .cmb_notification_list_row import CmbNotificationListRow
from .cmb_notification import CmbNotificationCenter, CmbVersionNotification, CmbMessageNotification, CmbPollNotification

logger = getLogger(__name__)


@Gtk.Template(resource_path="/ar/xjuan/Cambalache/cmb_notification_list_view.ui")
class CmbNotificationListView(Gtk.Box):
    __gtype_name__ = "CmbNotificationListView"

    notification_center = GObject.Property(type=CmbNotificationCenter, flags=GObject.ParamFlags.READWRITE)

    list_box = Gtk.Template.Child()

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        self.connect("notify::notification-center", self.__on_notification_center_notify)

    def __on_notification_center_notify(self, obj, pspec):
        self.list_box.bind_model(self.notification_center.store, self.__create_widget_func)
        GObject.Object.bind_property(
            self.notification_center.store,
            "n-items",
            self.list_box,
            "visible",
            GObject.BindingFlags.SYNC_CREATE,
        )

    def __create_widget_func(self, item):
        if isinstance(item, CmbVersionNotification):
            view = CmbVersionNotificationView(notification=item)
        elif isinstance(item, CmbMessageNotification):
            view = CmbMessageNotificationView(notification=item)
        elif isinstance(item, CmbPollNotification):
            view = CmbPollNotificationView(notification=item)

        return CmbNotificationListRow(view=view)



#
# CmbGResourceEditor - Cambalache GResource Editor
#
# Copyright (C) 2024  Juan Pablo Ugarte
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation;
# version 2.1 of the License.
#
# library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
# Authors:
#   Juan Pablo Ugarte <juanpablougarte@gmail.com>
#
# SPDX-License-Identifier: LGPL-2.1-only
#

import os
from gi.repository import GObject, Gtk

from .cmb_gresource import CmbGResource
from cambalache import getLogger

logger = getLogger(__name__)


@Gtk.Template(resource_path="/ar/xjuan/Cambalache/cmb_gresource_editor.ui")
class CmbGResourceEditor(Gtk.Box):
    __gtype_name__ = "CmbGResourceEditor"

    stack = Gtk.Template.Child()

    gresources_filename = Gtk.Template.Child()
    gresource_prefix = Gtk.Template.Child()
    file_filename = Gtk.Template.Child()
    file_compressed = Gtk.Template.Child()
    file_preprocess = Gtk.Template.Child()
    file_alias = Gtk.Template.Child()

    fields = [
        ("gresources", "gresources_filename", "cmb-value"),
        ("gresource", "gresource_prefix", "cmb-value"),
        ("file", "file_filename", "cmb-value"),
        ("file", "file_compressed", "active"),
        ("file", "file_preprocess", "cmb-value"),
        ("file", "file_alias", "cmb-value"),
    ]

    def __init__(self, **kwargs):
        self._object = None
        self._bindings = []

        super().__init__(**kwargs)

    @GObject.Property(type=CmbGResource)
    def object(self):
        return self._object

    @object.setter
    def _set_object(self, obj):
        if obj == self._object:
            return

        for binding in self._bindings:
            binding.unbind()

        self._bindings = []
        self._object = obj

        if obj is None:
            self.set_sensitive(False)
            for for_type, field, target in self.fields:
                widget = getattr(self, field)
                target_prop = getattr(widget, target)

                if isinstance(target_prop, int):
                    setattr(widget, target, 0)
                else:
                    setattr(widget, target, None)
            return

        resource_type = obj.resource_type
        self.set_sensitive(True)
        self.stack.set_visible_child_name(resource_type)
        self.__update_gresources_bundle_dirname()

        for for_type, field, target in self.fields:
            if resource_type != for_type:
                continue

            binding = GObject.Object.bind_property(
                obj,
                field,
                getattr(self, field),
                target,
                GObject.BindingFlags.SYNC_CREATE | GObject.BindingFlags.BIDIRECTIONAL,
            )
            self._bindings.append(binding)

    def __update_gresources_bundle_dirname(self):
        filename = self._object.gresources_bundle.gresources_filename or self._object.project.filename or "."
        fullpath, relpath = self._object.project.get_abs_path(os.path.dirname(filename))
        self.gresources_filename.dirname = fullpath
        self.file_filename.dirname = fullpath

    @Gtk.Template.Callback("on_add_gresource_button_clicked")
    def __on_add_gresource_button_clicked(self, button):
        self._object.project.add_gresource("gresource", parent_id=self._object.gresource_id)

    @Gtk.Template.Callback("on_add_file_button_clicked")
    def __on_add_file_button_clicked(self, button):
        self._object.project.add_gresource("file", parent_id=self._object.gresource_id)


Gtk.WidgetClass.set_css_name(CmbGResourceEditor, "CmbGResourceEditor")



#
# Cambalache CSS wrapper
#
# Copyright (C) 2022  Juan Pablo Ugarte
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation;
# version 2.1 of the License.
#
# library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
# Authors:
#   Juan Pablo Ugarte <juanpablougarte@gmail.com>
#
# SPDX-License-Identifier: LGPL-2.1-only
#

import os

from gi.repository import GObject

from .cmb_path import CmbPath
from .cmb_base_objects import CmbBaseCSS

from cambalache import _


class CmbCSS(CmbBaseCSS):
    __gtype_name__ = "CmbCSS"

    __gsignals__ = {
        "parsing-status": (GObject.SignalFlags.RUN_FIRST, None, (int, str, int, int)),
    }

    path_parent = GObject.Property(type=CmbPath, flags=GObject.ParamFlags.READWRITE)

    def __init__(self, **kwargs):
        self._path = None

        super().__init__(**kwargs)
        self.update_file_monitor(self.filename)

        self.connect("notify", self.__on_notify)

    def _emit_parsing_status(self, error_count, message, start, end):
        self.emit("parsing-status", error_count, message, start, end)

    def __on_notify(self, obj, pspec):
        if pspec.name not in ["display-name", "file-status", "path-parent"]:
            self.project._css_changed(self, pspec.name)

        # Update display name if one of the following properties changed
        if pspec.name in ["filename", "file-status"]:
            self.notify("display-name")

        if pspec.name == "filename":
            self.update_file_monitor(self.filename)

    @classmethod
    def get_display_name(cls, css_id, filename):
        return os.path.basename(filename) if filename else _("Unnamed CSS {css_id}").format(css_id=css_id)

    @GObject.Property(type=str)
    def display_name(self):
        display_name = CmbCSS.get_display_name(self.css_id, self.filename)
        return f'<span underline="error">{display_name}</span>' if self.file_status else display_name

    @GObject.Property(type=int)
    def priority(self):
        retval = self.db_get("SELECT priority FROM css WHERE css_id=?;", (self.css_id,))
        return retval if retval is not None else 0

    @priority.setter
    def _set_priority(self, value):
        self.db_set("UPDATE css SET priority=? WHERE css_id=?;", (self.css_id,), value if value != 0 else None)

    @GObject.Property(type=object)
    def provider_for(self):
        c = self.project.db.cursor()

        retval = []
        for row in c.execute("SELECT ui_id FROM css_ui WHERE css_id=? ORDER BY ui_id;", (self.css_id,)):
            retval.append(row[0])

        c.close()
        return retval

    def reload(self):
        if not self.project or not self.filename:
            return False

        dirname = os.path.dirname(self.project.filename)
        path = os.path.join(dirname, self.filename)

        if os.path.exists(path):
            self._path = path

            with open(path) as fd:
                # Disable history
                self.project.history_enabled = False

                self.css = fd.read()
                fd.close()

                super().reload()

                return True
        else:
            self._path = None

        return False

    def add_ui(self, ui):
        c = self.project.db.cursor()

        # Do not use REPLACE INTO, to make sure both INSERT and UPDATE triggers are used
        count = self.db_get("SELECT count(css_id) FROM css_ui WHERE css_id=? AND ui_id=?;", (self.css_id, ui.ui_id))

        if count == 0:
            c.execute("INSERT INTO css_ui (css_id, ui_id) VALUES (?, ?);", (self.css_id, ui.ui_id))

        c.close()

        self.notify("provider_for")

    def remove_ui(self, ui):
        c = self.project.db.cursor()

        c.execute("DELETE FROM css_ui WHERE css_id=? AND ui_id=?;", (self.css_id, ui.ui_id))
        c.close()

        self.notify("provider_for")



#!/usr/bin/pytest

import os

from cambalache import CmbProject, config


def test_cmb_object_truth_value():
    project = CmbProject(target_tk="gtk-4.0")
    assert (project)

    ui = project.add_ui("liststore.ui")
    assert (ui)

    win = project.add_object(ui.ui_id, "GtkWindow")
    assert (win)


def cmb_object_data_test(target_tk):
    # Load original UI file to compare
    path = os.path.join(os.path.dirname(__file__), target_tk, "liststore.ui")
    str_original = open(path, "r").read()

    # Create new project to recreate 'liststore.ui' using API
    project = CmbProject(target_tk=target_tk)

    # Add an UI
    ui = project.add_ui("liststore.ui")

    if target_tk == "gtk+-3.0":
        ui.set_library("gtk+", "3.24")
    else:
        ui.set_library("gtk", "4.0")

    # Create a GtkListStore
    store = project.add_object(ui.ui_id, "GtkListStore")

    # Set object id
    store.name = "liststore_test"

    # Add columns
    cols = store.add_data("columns")
    for t, c in [
        ("gchararray", " column-name gchararray1 "),
        ("gint64", " column-name gint1 "),
        ("gboolean", " column-name gboolean1 "),
    ]:
        col = cols.add_data("column", comment=c)
        col.set_arg("type", t)

    # Add data
    data = store.add_data("data")

    for values in [("Hola", 1, False), ("Mundo", 2, True), ("Hello", 12, True), ("World", 1234, False)]:
        row = data.add_data("row")

        for i, val in enumerate(values):
            col = row.add_data("col", val)
            col.set_arg("id", i)

        # Add and remove data to CmbObjectData
        dummy_row = data.add_data("row")
        data.remove_data(dummy_row)

    # Add and remove data to CmbObject
    data2 = store.add_data("data")
    store.remove_data(data2)

    # Export UI file
    str_exported = project.db.tostring(ui.ui_id)

    # Remove "Created with" comment since version will not match
    str_original = "\n".join([ l for l in str_original.splitlines() if not l.startswith(f"<!-- Created with Cambalache")])
    str_exported = "\n".join([ l for l in str_exported.splitlines() if not l.startswith(f"<!-- Created with Cambalache")])

    assert str_exported == str_original


def test_gtk3_cmb_object_data():
    cmb_object_data_test("gtk+-3.0")


def test_gtk4_cmb_object_data():
    cmb_object_data_test("gtk-4.0")



#
# Merengue utilities
#
# Copyright (C) 2021  Juan Pablo Ugarte
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation;
# version 2.1 of the License.
#
# library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
# Authors:
#   Juan Pablo Ugarte <juanpablougarte@gmail.com>
#
# SPDX-License-Identifier: LGPL-2.1-only
#

import sys
import ctypes

from ctypes.util import find_library
from gi.repository import Gdk, Gtk


def object_get_builder_id(obj):
    if obj is None:
        return None

    if Gtk.MAJOR_VERSION == 4:
        if issubclass(type(obj), Gtk.Buildable):
            return Gtk.Buildable.get_buildable_id(obj)
        else:
            return _g_object_get_data(obj, "gtk-builder-id")
    else:
        if issubclass(type(obj), Gtk.Buildable):
            return Gtk.Buildable.get_name(obj)
        else:
            return _g_object_get_data(obj, "gtk-builder-name")


def object_get_id(obj):
    if obj is None:
        return None

    builder_id = object_get_builder_id(obj)
    if builder_id and builder_id.startswith("__cmb__"):
        return builder_id[7:]

    return None


def gesture_click_new(widget, **kwargs):
    if Gtk.MAJOR_VERSION == 4:
        retval = Gtk.GestureClick(**kwargs)
        widget.add_controller(retval)
    else:
        widget.add_events(Gdk.EventMask.BUTTON_PRESS_MASK | Gdk.EventMask.BUTTON_RELEASE_MASK)
        retval = Gtk.GestureMultiPress(widget=widget, **kwargs)
    return retval


def scroll_controller_new(widget, **kwargs):
    if Gtk.MAJOR_VERSION == 4:
        retval = Gtk.EventControllerScroll(**kwargs)
        widget.add_controller(retval)
    else:
        widget.add_events(Gdk.EventMask.SCROLL_MASK)
        retval = Gtk.EventControllerScroll(widget=widget, **kwargs)
    return retval


#
# CTYPES HACKS
#

# Python object size, needed to calculate GObject pointer
object_size = sys.getsizeof(object())


# Return C object pointer
def _cobject(obj):
    return ctypes.c_void_p.from_address(id(obj) + object_size)


# GObject library
gobject = ctypes.CDLL(find_library("gobject-2.0"))

# g_object_get_data
gobject.g_object_get_data.argtypes = [ctypes.c_void_p, ctypes.c_char_p]
gobject.g_object_get_data.restype = ctypes.c_char_p


def _g_object_get_data(obj, key):
    retval = gobject.g_object_get_data(_cobject(obj), key.encode("utf-8"))
    return retval.decode("utf-8") if retval else None



#
# CmbBooleanUndefined
#
# Copyright (C) 2024  Juan Pablo Ugarte
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation;
# version 2.1 of the License.
#
# library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
# Authors:
#   Juan Pablo Ugarte <juanpablougarte@gmail.com>
#
# SPDX-License-Identifier: LGPL-2.1-only
#

from gi.repository import GObject, Gtk
from cambalache import _


class CmbBooleanUndefined(Gtk.Box):
    __gtype_name__ = "CmbBooleanUndefined"

    def __init__(self, **kwargs):
        super().__init__(**kwargs)

        self.__undefined = Gtk.CheckButton(label=_("undefined"))
        self.__true = Gtk.CheckButton(label=_("true"), group=self.__undefined)
        self.__false = Gtk.CheckButton(label=_("false"), group=self.__undefined)

        for button in [self.__undefined, self.__true, self.__false]:
            self.append(button)
            button.connect("notify::active", self.__on_notify)

    def __on_notify(self, obj, pspec):
        self.notify("cmb-value")

    @GObject.Property(type=str)
    def cmb_value(self):
        if self.__undefined.props.active:
            return "undefined"
        elif self.__true.props.active:
            return "True"

        return "False"

    @cmb_value.setter
    def _set_cmb_value(self, value):
        if value is not None:
            if type(value) is str:
                val = value.lower()

                if val == "undefined":
                    self.__undefined.props.active = True
                    return

                if val in {"1", "t", "y", "true", "yes"}:
                    active = True
                else:
                    active = False
            else:
                active = bool(value)

            if active:
                self.__true.props.active = True
            else:
                self.__false.props.active = True
        else:
            self.__undefined.active = True
